/*
 * Copyright (c) 2004-2012 The YAWL Foundation. All rights reserved.
 * The YAWL Foundation is a collaboration of individuals and
 * organisations who are committed to improving workflow technology.
 *
 * This file is part of YAWL. YAWL is free software: you can
 * redistribute it and/or modify it under the terms of the GNU Lesser
 * General Public License as published by the Free Software Foundation.
 *
 * YAWL is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
 * Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with YAWL. If not, see <http://www.gnu.org/licenses/>.
 */

package org.yawlfoundation.yawl.engine;

import java.util.Map;
import java.util.HashMap;

/**
 * @author Mike Fowler
 *         Date: 16-Oct-2006
 */
public enum YWorkItemStatus {

    statusEnabled("Enabled"),
    statusFired("Fired"),
    statusExecuting("Executing"),
    statusComplete("Complete"),
    statusIsParent("Is parent"),
    statusDeadlocked("Deadlocked"),
    statusDeleted("Cancelled"),                       // by cancel set or exception
    statusWithdrawn("Withdrawn"),                     // by deferred choice etc
	statusForcedComplete("ForcedComplete"),
	statusFailed("Failed"),
	statusSuspended("Suspended"),
    statusCancelledByCase("CancelledByCase"),
    statusDiscarded("Discarded");                     // tokens in net after completion

    private String statusString;


    private static final Map<String, YWorkItemStatus> _fromStringMap =
            new HashMap<String, YWorkItemStatus>(13);

    static {
        for (YWorkItemStatus itemStatus : values()) {
            _fromStringMap.put(itemStatus.toString(), itemStatus);
        }
    }


    YWorkItemStatus(String status) {
        statusString = status;
    }

    @Override
    public String toString() {
        return statusString;
    }

    public static YWorkItemStatus fromString(String s) {
        return (s != null) ? _fromStringMap.get(s) : null;
    }

}